/* ************************************************************************
*   File: config.c                                      Part of CircleMUD *
*  Usage: Configuration of various aspects of CircleMUD operation         *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

#define __CONFIG_CC__

#include "structs.h"
#include "awake.h"

#define YES     1
#define NO      0

/*
 * Below are several constants which you can change to alter certain aspects
 * of the way CircleMUD acts.  Since this is a .c file, all you have to do
 * to change one of the constants (assuming you keep your object files around)
 * is change the constant in this file and type 'make'.  Make will recompile
 * this file and relink; you don't have to wait for the whole thing to
 * recompile as you do if you change a header file.
 *
 * I realize that it would be slightly more efficient to have lots of
 * #defines strewn about, so that, for example, the autowiz code isn't
 * compiled at all if you don't want to use autowiz.  However, the actual
 * code for the various options is quite small, as is the computational time
 * in checking the option you've selected at run-time, so I've decided the
 * convenience of having all your options in this one file outweighs the
 * efficency of doing it the other way.
 *
 */

/****************************************************************************/

/****************************************************************************/
/* Do you want to automatically wipe players who've been gone too long? */
int auto_pwipe = NO;

/* Autowipe deletion criteria
   This struct holds information used to determine which players to wipe
   then the mud boots.  The levels must be in ascending order, with a
   descending level marking the end of the array.  A level -1 entry in the
   beginning is the case for players with the PLR_DELETED flag.  The
   values below match the stock purgeplay.c criteria.

   Detailed explanation by array element:
   * Element 0, level -1, days 0: Players with PLR_DELETED flag are always
	wiped.
   * Element 1, level 0, days 0: Players at level 0 have created a
	character, but have never actually entered the game, so always
	wipe them.
   * Element 2, level 1, days 4: Players at level 1 are wiped if they
	haven't logged on in the past 4 days.
   * Element 3, level 4, days 7: Players level 2 through 4 are wiped if
	they haven't logged on in the past 7 days.
   * Element 4, level 10, days 30: Players level 5-10 get 30 days.
   * Element 5, level LVL_IMMORT - 1, days 60: All other mortals get
	60 days.
   * Element 6, level LVL_IMPL, days 90: Immortals get 90 days.
   * Element 7: Because -2 is less than LVL_IMPL, this is assumed to
	be the end of the criteria.  The days entry is not used in this
	case.
*/
struct pclean_criteria_data pclean_criteria[] = {
/*	LEVEL		DAYS	*/
  {	-1		,0	}, /* players with PLR_DELETE flag */
  {	0		,30	}, /* level 0 */
  {	1		,60	}, /* players (mortal) who have not logged 
on in 30 days*/
  {	LVL_PRESIDENT -1, 180	}, /* all immortals below president get 3 
months */
  {	-2		,0	}  /* no more level checks */
};

/* Do you want players who self-delete to be wiped immediately with no
   backup?
*/
int selfdelete_fastwipe = YES;

/* GAME PLAY OPTIONS */

/*
 * pk_allowed sets the tone of the entire game.  If pk_allowed is set to
 * NO, then players will not be allowed to kill, summon, charm, or sleep
 * other players, as well as a variety of other "asshole player" protections.
 * However, if you decide you want to have an all-out knock-down drag-out
 * PK Mud, just set pk_allowed to YES - and anything goes.
 */
int pk_allowed = NO;

/* is playerthieving allowed? */
int pt_allowed = NO;

/* minimum level a player must be to shout/holler/gossip/auction */
int level_can_shout = 0;

/* number of movement points it costs to holler */
int holler_move_cost = 20;

/* exp change limits */
int max_exp_gain = 250; /* max gainable per kill */
int max_exp_loss = 1000;        /* max losable per death */

/* number of tics (usually 75 seconds) before PC/NPC corpses decompose */
int max_npc_corpse_time = 3;
int max_pc_corpse_time = 10;

/* should items in death traps automatically be junked? */
int dts_are_dumps = YES;

/* "okay" etc. */
char *OK = "Okay.\r\n";
char *NOPERSON = "No-one by that name here.\r\n";
char *NOEFFECT = "Nothing seems to happen.\r\n";

/****************************************************************************/
/****************************************************************************/


/* RENT/CRASHSAVE OPTIONS */

/*
 * Should the MUD allow you to 'rent' for free?  (i.e. if you just quit,
 * your objects are saved at no cost, as in Merc-type MUDs.
 */
int free_rent = NO;

/* maximum number of items players are allowed to rent */
int max_obj_save = 150;

/* receptionist's surcharge on top of item costs */
int min_rent_cost = 100;

/*
 * Should the game automatically save people?  (i.e., save player data
 * every 4 kills (on average), and Crash-save as defined below.
 */
int auto_save = YES;

/*
 * if auto_save (above) is yes, how often (in minutes) should the MUD
 * Crash-save people's objects?   Also, this number indicates how often
 * the MUD will Crash-save players' houses.
 */
int autosave_time = 5;

/* Lifetime of crashfiles and forced-rent (idlesave) files in days */
int crash_file_timeout = 30;

/* Lifetime of normal rent files in days */
int rent_file_timeout = 60;


/****************************************************************************/
/****************************************************************************/


/* ROOM NUMBERS */

/* virtual number of room that newbies should enter at */
sh_int newbie_start_room = 8000;

/* virtual number of room that mortals should enter at */
sh_int mortal_start_room = 2000;

/* virtual number of room that immorts should enter at by default */
sh_int immort_start_room = 1000;

/* virtual number of room that frozen players should enter at */
sh_int frozen_start_room = 1050;

/*
 * virtual numbers of donation rooms.  note: you must change code in
 * do_drop of act.obj1.c if you change the number of non-NOWHERE
 * donation rooms.
 */
sh_int donation_room_1 = 8047;
sh_int donation_room_2 = 8048;
sh_int donation_room_3 = 8049;
sh_int donation_room_4 = 8050;
sh_int donation_room_5 = 8051;
sh_int donation_room_6 = 8052;

//sh_int donation_room_2 = NOWHERE;     /* unused - room for expansion */
//sh_int donation_room_3 = NOWHERE;     /* unused - room for expansion */


/****************************************************************************/
/****************************************************************************/


/* GAME OPERATION OPTIONS */

/* default port the game should run on if no port given on command-line */
int DFLT_PORT = 2050;

/* default directory to use as data directory */
char *DFLT_DIR = "lib";

/* maximum number of players allowed before game starts to turn people away */
int MAX_PLAYERS = 300;

/* maximum size of bug, typo and idea files (to prevent bombing) */
int max_filesize = 50000;

/* maximum number of password attempts before disconnection */
int max_bad_pws = 3;

/*
 * Some nameservers are very slow and cause the game to lag terribly every
 * time someone logs in.  The lag is caused by the gethostbyaddr() function
 * which is responsible for resolving numeric IP addresses to alphabetic names.
 * Sometimes, nameservers can be so slow that the incredible lag caused by
 * gethostbyaddr() isn't worth the luxury of having names instead of numbers
 * for players' sitenames.
 *
 * If your nameserver is fast, set the variable below to NO.  If your
 * nameserver is slow, of it you would simply prefer to have numbers
 * instead of names for some other reason, set the variable to YES.
 *
 * You can experiment with the setting of nameserver_is_slow on-line using
 * the SLOWNS command from within the MUD.
 */

int nameserver_is_slow = YES;

char *MENU =
"\r\n"
"^YWelcome ^Cto ^BAwake 2050^n!\r\n"
"^R0^n) Exit from ^BAwake 2050^n.\r\n"
"^G1^n) Enter the game.\r\n"
"^G2^n) Enter description.\r\n"
"^G3^n) Read the background story.\r\n"
"^G4^n) Check rent status.\r\n"
"^G5^n) Change password.\r\n"
"^L6^n) Delete this character.\r\n"
"\r\n"
"   Make your choice: ";

char *QMENU =
"\r\n"
"Current options:\r\n"
"^R0^n) Exit from ^BAwake 2050^n.\r\n"
"^G1^n) Check rent status.\r\n"
"^G2^n) Change password.\r\n"
"^G3^n) Delete this character.\r\n"
"\r\n"
"   Make your choice: ";

char *GREETINGS =
"\r\n"
"Awake 2050/AwakeMUD Code Development Port\r\n"
"Based on CircleMUD 3.0 by Jeremy Elson.\r\n"
"Derived from DikuMUD (GAMMA 0.0) by Hans Henrik Staerfeld, Katja Nyboe,\r\n"
"Tom Madsen, Michael Sefert, and Sebastian Hammer.\r\n"
"AwakeMUD 0.61 Beta by Flynn, Rift, Fastjack, and Washu.\r\n"
"Enter your login name:\r\n" ;

char *WELC_MESSG =
"\r\n"
"Welcome to the future, 2050, the Sixth World to some, an Awakening to all.\r\n"
"\r\n\r\n";

char *START_MESSG =
"Welcome to the future, 2050, where mankind has entered what the Mayans would\r\n"
"call the Sixth World.  New races, magic, and technology all clash in what we\r\n"
"call Awake 2050.  If you have never experienced this world before, typing\r\n"
"help NEWBIE will help you understand what exactly is going on here.\r\n\r\n";


/****************************************************************************/
/****************************************************************************/


/* AUTOWIZ OPTIONS */

/* Should the game automatically create a new wizlist/immlist every time
   someone immorts, or is promoted to a higher (or lower) god level? */
int use_autowiz = YES;

/* If yes, what is the lowest level which should be on the wizlist?  (All
   immort levels below the level you specify will go on the immlist instead.) */
int min_wizlist_lev = LVL_MANAGER;

/* If YES, the mud will attempt to find the remote user name for each
   player connecting to the mud.  This can also be toggled in the game
   with the "ident" command. */
int ident = YES;
